#include "MPC5604B_M27V.h"

int32_t temp1,temp2;
int32_t period[10];
int32_t cnt = 0;


void initModesAndClock(void) {
  ME.MER.R = 0x0000001D;          /* Enable DRUN, RUN0, SAFE, RESET modes */
                                  /* Initialize PLL before turning it on: */
/* Use one of the next two lines depending on crystal frequency: */
  CGM.FMPLL_CR.R = 0x02400100; /* 8 MHz xtal: Set PLL0 to 64 MHz */   
/* CGM.FMPLL[0].CR.R = 0x12400100;*/ /* 40 MHz xtal: Set PLL0 to 64 MHz */   
  ME.RUN[0].R = 0x001F0074;       /* RUN0 cfg: 16MHzIRCON,OSC0ON,PLL0ON,syclk=PLL0 */
  ME.RUNPC[0].R = 0x00000010; 	  /* Peri. Cfg. 0 settings: only run in RUN0 mode */
                                  /* Mode Transition to enter RUN0 mode: */
  ME.MCTL.R = 0x40005AF0;         /* Enter RUN0 Mode & Key */
  ME.MCTL.R = 0x4000A50F;         /* Enter RUN0 Mode & Inverted Key */  
  while (ME.IS.B.I_MTC != 1) {}    /* Wait for mode transition to complete */    
                                  /* Notes: */
                                  /* 1. I_TC IRQ could be used here instead of polling */
                                  /*    to allow software to complete other init. */
                                  /* 2. A timer could be used to prevent waiting forever.*/  
  while(ME.GS.B.S_CURRENTMODE != 4) {} /* Verify target mode RUN0 is the current mode */
  ME.IS.R = 0x00000001;                /* Clear Mode Transition Complete flag */    
}

void initEMIOS_0(void) {
  EMIOS_0.MCR.B.GPRE= 63;     /* Divide 64 MHz sysclk by 63+1 = 64 for 1MHz eMIOS clk*/
  EMIOS_0.MCR.B.ETB = 0;      /* External time base is disabled; Ch 23 drives ctr bus A */
  EMIOS_0.MCR.B.GPREN = 1;	  /* Disable eMIOS clock */
  EMIOS_0.MCR.B.GTBE = 1;		  /* Disable global time base */
  EMIOS_0.MCR.B.FRZ = 1;		  /* Enable stopping channels when in debug mode */
}

void initEMIOS_0ch23(void) {        /* EMIOS 0 CH 23: Modulus Up Counter */
  EMIOS_0.CH[23].CADR.R = 999;      /* Period will be 999+1 = 1000 clocks (1 msec) */
  EMIOS_0.CH[23].CCR.B.MODE = 0x50; /* Modulus Counter Buffered (MCB) */
  EMIOS_0.CH[23].CCR.B.BSL = 0x3;	  /* Use internal counter */
  EMIOS_0.CH[23].CCR.B.UCPRE=0;	    /* Set channel prescaler to divide by 1 */
  EMIOS_0.CH[23].CCR.B.UCPEN = 1;   /* Enable prescaler; uses default divide by 1 */
  EMIOS_0.CH[23].CCR.B.FREN = 1; 	  /* Freeze channel counting when in debug mode */
}


/*Input Pulse width measurement */
void initEMIOS_0ch12(void)
{
  EMIOS_0.CH[12].CCR.B.BSL = 0x0;	  /* Use counter bus A (default) */
  EMIOS_0.CH[12].CCR.B.EDPOL = 1;	  /* positive edge */ 
  EMIOS_0.CH[12].CCR.B.MODE = 0x04; /* mode: IPWM */
  EMIOS_0.CH[12].CCR.B.FEN = 1;     /*enable interrupt */        
  
  SIU.PCR[44].R = 0x0500;        /* Initialize pad for eMIOS chan. input */
}

void INT_eMIOS_0ch12(void)
{
		EMIOS_0.CH[12].CSR.B.FLAG = 1;   /*claer the flag*/	
		temp1 = EMIOS_0.CH[12].CADR.R;
		temp2 = EMIOS_0.CH[12].CBDR.R;		
		
		//save first 10 values into "period"
		period[cnt++] = temp1-temp2;
    if(cnt>9)
      EMIOS_0.CH[12].CCR.B.FEN = 0;  /*disable interrupt */		
      
    //overflow is not handled in this example  
}


int main(void) {
  volatile int i = 0;

  initModesAndClock();  /* Initialize mode entries and system clock */
  
  
  initEMIOS_0();        /* Initialize eMIOS 0 to provide 1 MHz clock to channels */
  initEMIOS_0ch23();    /* Initialize eMIOS 0 channel 23 as modulus counter*/
  initEMIOS_0ch12();
  
  INTC_InstallINTCInterruptHandler(INT_eMIOS_0ch12,147,3);
  
  INTC.CPR.B.PRI = 0;	
  
  /* Loop forever */
  for (;;) {      
    i++;
  }
}



